(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('lodash')) :
    typeof define === 'function' && define.amd ? define(['exports', 'lodash'], factory) :
    (factory((global.TypeMoq = global.TypeMoq || {}),global._));
}(this, (function (exports,_) {

(function (MockBehavior) {
    MockBehavior[MockBehavior["Loose"] = 0] = "Loose";
    MockBehavior[MockBehavior["Strict"] = 1] = "Strict";
})(exports.MockBehavior || (exports.MockBehavior = {}));

(function (ExpectedCallType) {
    ExpectedCallType[ExpectedCallType["InAnyOrder"] = 0] = "InAnyOrder";
    ExpectedCallType[ExpectedCallType["InSequence"] = 1] = "InSequence";
})(exports.ExpectedCallType || (exports.ExpectedCallType = {}));

var Times = (function () {
    function Times(_condition, min, max, failMessage) {
        this._condition = _condition;
        this.min = min;
        this.max = max;
        this._failMessage = _.template(failMessage);
    }
    Times.prototype.failMessage = function (call) {
        return this._failMessage({ i: call, n: this.min, m: this._lastCallCount });
    };
    Times.prototype.verify = function (callCount) {
        this._lastCallCount = callCount;
        return this._condition(callCount);
    };
    Times.exactly = function (n) {
        return new Times(function (c) { return c === n; }, n, n, Times.NO_MATCHING_CALLS_EXACTLY_N_TIMES);
    };
    Times.never = function () {
        return Times.exactly(0);
    };
    Times.once = function () {
        return Times.exactly(1);
    };
    Times.atLeastOnce = function () {
        return new Times(function (c) { return c >= 1; }, 1, 255, Times.NO_MATCHING_CALLS_AT_LEAST_ONCE);
    };
    Times.atMostOnce = function () {
        return new Times(function (c) { return c >= 0 && c <= 1; }, 0, 1, Times.NO_MATCHING_CALLS_AT_MOST_ONCE);
    };
    return Times;
}());
Times.NO_MATCHING_CALLS_EXACTLY_N_TIMES = "expected invocation of <%= i %> <%= n %> times, invoked <%= m %> times";
Times.NO_MATCHING_CALLS_AT_LEAST_ONCE = "expected invocation of <%= i %> at least once, invoked <%= m %> times";
Times.NO_MATCHING_CALLS_AT_MOST_ONCE = "expected invocation of <%= i %> at most once, invoked <%= m %> times";

var PropertyRetriever = (function () {
    function PropertyRetriever() {
    }
    PropertyRetriever.getOwnEnumerables = function (obj) {
        return this._getPropertyNames(obj, true, false, this._enumerable);
    };
    PropertyRetriever.getOwnNonenumerables = function (obj) {
        return this._getPropertyNames(obj, true, false, this._notEnumerable);
    };
    PropertyRetriever.getOwnEnumerablesAndNonenumerables = function (obj) {
        return this._getPropertyNames(obj, true, false, this._enumerableAndNotEnumerable);
    };
    PropertyRetriever.getPrototypeEnumerables = function (obj) {
        return this._getPropertyNames(obj, false, true, this._enumerable);
    };
    PropertyRetriever.getPrototypeNonenumerables = function (obj) {
        return this._getPropertyNames(obj, false, true, this._notEnumerable);
    };
    PropertyRetriever.getPrototypeEnumerablesAndNonenumerables = function (obj) {
        return this._getPropertyNames(obj, false, true, this._enumerableAndNotEnumerable);
    };
    PropertyRetriever.getOwnAndPrototypeEnumerables = function (obj) {
        return this._getPropertyNames(obj, true, true, this._enumerable);
    };
    PropertyRetriever.getOwnAndPrototypeNonenumerables = function (obj) {
        return this._getPropertyNames(obj, true, true, this._notEnumerable);
    };
    PropertyRetriever.getOwnAndPrototypeEnumerablesAndNonenumerables = function (obj) {
        return this._getPropertyNames(obj, true, true, this._enumerableAndNotEnumerable);
    };
    PropertyRetriever._enumerable = function (obj, prop) {
        return obj.propertyIsEnumerable(prop);
    };
    PropertyRetriever._notEnumerable = function (obj, prop) {
        return !obj.propertyIsEnumerable(prop);
    };
    PropertyRetriever._enumerableAndNotEnumerable = function (obj, prop) {
        return true;
    };
    PropertyRetriever._getPropertyNames = function (obj, iterateSelfBool, iteratePrototypeBool, includePropCb) {
        var result = [];
        do {
            if (iterateSelfBool) {
                var props = Object.getOwnPropertyNames(obj);
                var _loop_1 = function (prop) {
                    var duplicate = _.find(result, function (p) { return p.name === prop; });
                    if (!duplicate && includePropCb(obj, prop)) {
                        var propDesc = Object.getOwnPropertyDescriptor(obj, prop);
                        result.push({ name: prop, desc: propDesc });
                    }
                };
                for (var _i = 0, props_1 = props; _i < props_1.length; _i++) {
                    var prop = props_1[_i];
                    _loop_1(prop);
                }
                
            }
            if (!iteratePrototypeBool) {
                break;
            }
            iterateSelfBool = true;
        } while (obj = Object.getPrototypeOf(obj));
        return result;
    };
    return PropertyRetriever;
}());

var Utils = (function () {
    function Utils() {
    }
    Utils.functionName = function (fun) {
        var res;
        if (fun.name) {
            res = fun.name;
        }
        else {
            var repr = fun.toString();
            repr = repr.substr('function '.length);
            res = repr.substr(0, repr.indexOf('('));
        }
        return res;
    };
    Utils.objectName = function (obj) {
        var res = Utils.functionName(obj.constructor);
        return res;
    };
    Utils.argsName = function (args) {
        var res = _.join(args);
        return res;
    };
    Utils.conthunktor = function (ctor, args) {
        var ret = new (ctor.bind.apply(ctor, [void 0].concat(args)))();
        return ret;
    };
    return Utils;
}());

var __extends$1 = (undefined && undefined.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Exception = (function (_super) {
    __extends$1(Exception, _super);
    function Exception(name, message) {
        var _this = _super.call(this, message) || this;
        _this.name = name;
        return _this;
    }
    Exception.prototype.toString = function () {
        var errMsg = this.message ? this.name + " - " + this.message : this.name;
        return errMsg;
    };
    return Exception;
}(Error));

var __extends$2 = (undefined && undefined.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var MockExceptionReason;
(function (MockExceptionReason) {
    MockExceptionReason[MockExceptionReason["NoSetup"] = "no setup expression found for"] = "NoSetup";
    MockExceptionReason[MockExceptionReason["MoreThanOneSetup"] = "more than one setup expression found for"] = "MoreThanOneSetup";
    MockExceptionReason[MockExceptionReason["InvalidSetup"] = "invalid setup expression"] = "InvalidSetup";
    MockExceptionReason[MockExceptionReason["InvalidMatcher"] = "invalid matching expression"] = "InvalidMatcher";
    MockExceptionReason[MockExceptionReason["InvalidProxyArg"] = "invalid proxy argument"] = "InvalidProxyArg";
    MockExceptionReason[MockExceptionReason["UnknownGlobalType"] = "unknown global type"] = "UnknownGlobalType";
    MockExceptionReason[MockExceptionReason["CallCountVerificationFailed"] = "invocation count verification failed"] = "CallCountVerificationFailed";
    MockExceptionReason[MockExceptionReason["CallOrderVerificationFailed"] = "invocation order verification failed"] = "CallOrderVerificationFailed";
    MockExceptionReason[MockExceptionReason["InvalidDynamicProxyRuntime"] = "invalid dynamic proxy runtime"] = "InvalidDynamicProxyRuntime";
})(MockExceptionReason || (MockExceptionReason = {}));
var MockException = (function (_super) {
    __extends$2(MockException, _super);
    function MockException(reason, ctx, message) {
        if (message === void 0) { message = undefined; }
        var _this = _super.call(this, "MockException", message) || this;
        _this.reason = reason;
        _this.ctx = ctx;
        _this.message = message;
        return _this;
    }
    MockException.prototype.toString = function () {
        var errMsg = this.name + " - " + this.reason;
        if (this.message)
            errMsg = errMsg + " (" + this.message + ")";
        return errMsg;
    };
    return MockException;
}(Exception));

var Consts = (function () {
    function Consts() {
    }
    return Consts;
}());
Consts.IMATCH_ID_VALUE = "438A51D3-6864-49D7-A655-CA1153B86965";
Consts.IMATCH_ID_NAME = "___id";
Consts.IMATCH_MATCHES_NAME = "___matches";
Consts.IPROXY_ID_VALUE = "BCDF5CE5-F0DF-40B7-8BA0-69DF395033C8";
Consts.IPROXY_ID_NAME = "___id";

var MatchAnyObject = (function () {
    function MatchAnyObject(_ctor) {
        this._ctor = _ctor;
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchAnyObject.prototype.___matches = function (object) {
        var match = false;
        if (this._ctor.prototype === object.constructor.prototype)
            match = true;
        return match;
    };
    MatchAnyObject.prototype.toString = function () {
        var res = "It.isAnyObject(" + Utils.functionName(this._ctor) + ")";
        return res;
    };
    return MatchAnyObject;
}());
var MatchAny = (function () {
    function MatchAny() {
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchAny.prototype.___matches = function (object) {
        var match = false;
        if (!_.isNil(object))
            match = true;
        return match;
    };
    MatchAny.prototype.toString = function () {
        return "It.isAny()";
    };
    return MatchAny;
}());
var MatchAnyString = (function () {
    function MatchAnyString() {
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchAnyString.prototype.___matches = function (object) {
        var match = false;
        if (_.isString(object))
            match = true;
        return match;
    };
    MatchAnyString.prototype.toString = function () {
        return "It.isAnyString()";
    };
    return MatchAnyString;
}());
var MatchAnyNumber = (function () {
    function MatchAnyNumber() {
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchAnyNumber.prototype.___matches = function (object) {
        var match = false;
        if (_.isNumber(object))
            match = true;
        return match;
    };
    MatchAnyNumber.prototype.toString = function () {
        return "It.isAnyNumber()";
    };
    return MatchAnyNumber;
}());

var MatchPred = (function () {
    function MatchPred(_pred) {
        this._pred = _pred;
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchPred.prototype.___matches = function (object) {
        var match = false;
        if (this._pred(object))
            match = true;
        return match;
    };
    MatchPred.prototype.toString = function () {
        var res = "It.is(" + this._pred + ")";
        return res;
    };
    return MatchPred;
}());

var MatchValue = (function () {
    function MatchValue(_value) {
        this._value = _value;
        this.___id = Consts.IMATCH_ID_VALUE;
    }
    MatchValue.prototype.___matches = function (object) {
        var match = false;
        if (_.isEqual(this._value, object))
            match = true;
        return match;
    };
    MatchValue.prototype.toString = function () {
        var res = "It.isValue(" + this._value + ")";
        return res;
    };
    return MatchValue;
}());

var MethodInvocation = (function () {
    function MethodInvocation(_property, _args) {
        this._property = _property;
        this._args = _args;
    }
    Object.defineProperty(MethodInvocation.prototype, "args", {
        get: function () { return this._args || { length: 0, callee: null }; },
        set: function (value) { this._args = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodInvocation.prototype, "property", {
        get: function () { return this._property; },
        enumerable: true,
        configurable: true
    });
    MethodInvocation.prototype.invokeBase = function () {
        this.returnValue = this._property.toFunc.apply(this._property.obj, this._args);
    };
    MethodInvocation.prototype.toString = function () {
        var res = this.property + "(" + Utils.argsName(this.args) + ")";
        return res;
    };
    return MethodInvocation;
}());
var ValueGetterInvocation = (function () {
    function ValueGetterInvocation(_property, value) {
        this._property = _property;
        this.value = value;
        this.returnValue = value;
    }
    Object.defineProperty(ValueGetterInvocation.prototype, "args", {
        get: function () {
            var args = [];
            Object.defineProperty(args, "callee", { configurable: false, enumerable: true, writable: false, value: null });
            return args;
        },
        set: function (value) { },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValueGetterInvocation.prototype, "property", {
        get: function () { return this._property; },
        enumerable: true,
        configurable: true
    });
    ValueGetterInvocation.prototype.invokeBase = function () {
        this.returnValue = this._property.obj[this._property.name];
    };
    ValueGetterInvocation.prototype.toString = function () {
        var res = "" + this.property;
        return res;
    };
    return ValueGetterInvocation;
}());
var ValueSetterInvocation = (function () {
    function ValueSetterInvocation(_property, _args) {
        this._property = _property;
        this._args = _args;
    }
    Object.defineProperty(ValueSetterInvocation.prototype, "args", {
        get: function () { return this._args; },
        set: function (value) { this._args = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ValueSetterInvocation.prototype, "property", {
        get: function () { return this._property; },
        enumerable: true,
        configurable: true
    });
    ValueSetterInvocation.prototype.invokeBase = function () {
        this._property.obj[this._property.name] = this._args[0];
        this.returnValue = this._property.obj[this._property.name];
    };
    ValueSetterInvocation.prototype.toString = function () {
        var res = this.property + " = " + Utils.argsName(this.args[0]);
        return res;
    };
    return ValueSetterInvocation;
}());
var MethodGetterInvocation = (function () {
    function MethodGetterInvocation(_property, _getter) {
        this._property = _property;
        this._getter = _getter;
    }
    Object.defineProperty(MethodGetterInvocation.prototype, "args", {
        get: function () {
            var args = [];
            Object.defineProperty(args, "callee", { configurable: false, enumerable: true, writable: false, value: null });
            return args;
        },
        set: function (value) { },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodGetterInvocation.prototype, "property", {
        get: function () { return this._property; },
        enumerable: true,
        configurable: true
    });
    MethodGetterInvocation.prototype.invokeBase = function () {
        this.returnValue = this._property.obj[this._property.name];
    };
    MethodGetterInvocation.prototype.toString = function () {
        var res = "" + this.property;
        return res;
    };
    return MethodGetterInvocation;
}());
var MethodSetterInvocation = (function () {
    function MethodSetterInvocation(_property, _setter, _args) {
        this._property = _property;
        this._setter = _setter;
        this._args = _args;
    }
    Object.defineProperty(MethodSetterInvocation.prototype, "args", {
        get: function () { return this._args; },
        set: function (value) { this._args = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodSetterInvocation.prototype, "property", {
        get: function () { return this._property; },
        enumerable: true,
        configurable: true
    });
    MethodSetterInvocation.prototype.invokeBase = function () {
        this._property.obj[this._property.name] = this._args[0];
        this.returnValue = this._property.obj[this._property.name];
    };
    MethodSetterInvocation.prototype.toString = function () {
        var res = this.property + "(" + Utils.argsName(this.args[0]) + ")";
        return res;
    };
    return MethodSetterInvocation;
}());
var MethodInfo = (function () {
    function MethodInfo(obj, name, desc) {
        this.obj = obj;
        this.name = name;
        this.desc = desc;
    }
    Object.defineProperty(MethodInfo.prototype, "toFunc", {
        get: function () {
            var func;
            if (_.isFunction(this.obj))
                func = this.obj;
            else
                func = this.obj[this.name];
            return func;
        },
        enumerable: true,
        configurable: true
    });
    MethodInfo.prototype.toString = function () {
        var objName = Utils.objectName(this.obj);
        var res = objName + "." + this.name;
        return res;
    };
    return MethodInfo;
}());
var PropertyInfo = (function () {
    function PropertyInfo(obj, name, desc) {
        this.obj = obj;
        this.name = name;
        this.desc = desc;
    }
    PropertyInfo.prototype.toString = function () {
        var objName = Utils.objectName(this.obj);
        var res = objName + "." + this.name;
        return res;
    };
    return PropertyInfo;
}());

var Proxy$1 = (function () {
    function Proxy(interceptor, instance) {
        var _this = this;
        this.___id = Consts.IPROXY_ID_VALUE;
        this.check(instance);
        var that = this;
        var props = PropertyRetriever.getOwnAndPrototypeEnumerablesAndNonenumerables(instance);
        _.each(props, function (prop) {
            if (_.isFunction(prop.desc.value)) {
                var propDesc = {
                    configurable: prop.desc.configurable,
                    enumerable: prop.desc.enumerable,
                    writable: prop.desc.writable
                };
                _this.defineMethodProxy(that, interceptor, instance, prop.name, propDesc);
            }
            else {
                var propDesc = {
                    configurable: prop.desc.configurable,
                    enumerable: prop.desc.enumerable
                };
                if (prop.desc.value !== undefined)
                    _this.defineValuePropertyProxy(that, interceptor, instance, prop.name, prop.desc.value, propDesc);
                else
                    _this.defineGetSetPropertyProxy(that, interceptor, instance, prop.name, prop.desc.get, prop.desc.set, propDesc);
            }
        });
    }
    Proxy.of = function (instance, interceptor) {
        Proxy.check(instance);
        var result;
        if (_.isFunction(instance)) {
            var funcName = Utils.functionName(instance);
            result = Proxy.methodProxyValue(interceptor, instance, funcName, null);
        }
        else {
            result = new Proxy(interceptor, instance);
        }
        return result;
    };
    Proxy.isProxy = function (obj) {
        if (!_.isNil(obj) &&
            !_.isUndefined(obj[Consts.IPROXY_ID_NAME]) && obj[Consts.IPROXY_ID_NAME] === Consts.IPROXY_ID_VALUE)
            return true;
        else
            return false;
    };
    Proxy.check = function (instance) {
        Proxy.checkNotNullOrUndefined(instance);
        var ok = false;
        if (_.isFunction(instance) ||
            (_.isObject(instance) && !Proxy.isPrimitiveObject(instance)))
            ok = true;
        if (!ok)
            throw new MockException(MockExceptionReason.InvalidProxyArg, instance, "'" + instance + "'; argument should be a function or a non primitive object");
    };
    Proxy.prototype.check = function (instance) {
        Proxy.checkNotNullOrUndefined(instance);
        var ok = false;
        if (!_.isFunction(instance) &&
            (_.isObject(instance) && !Proxy.isPrimitiveObject(instance)))
            ok = true;
        if (!ok)
            throw new MockException(MockExceptionReason.InvalidProxyArg, instance, "'" + instance + "'; argument should be a non primitive object");
    };
    Proxy.checkNotNullOrUndefined = function (instance) {
        if (_.isNil(instance))
            throw new MockException(MockExceptionReason.InvalidProxyArg, instance, "'" + instance + "'; argument is required");
    };
    Proxy.isPrimitiveObject = function (obj) {
        var result = false;
        if (_.isFunction(obj) ||
            _.isArray(obj) ||
            _.isDate(obj) ||
            _.isNull(obj))
            result = true;
        return result;
    };
    Proxy.prototype.defineMethodProxy = function (that, interceptor, instance, propName, propDesc) {
        if (propDesc === void 0) { propDesc = { configurable: false, enumerable: true, writable: false }; }
        propDesc.value = Proxy.methodProxyValue(interceptor, instance, propName, propDesc);
        this.defineProperty(that, propName, propDesc);
    };
    Proxy.methodProxyValue = function (interceptor, instance, propName, propDesc) {
        function proxy() {
            var method = new MethodInfo(instance, propName, propDesc);
            var invocation = new MethodInvocation(method, arguments);
            interceptor.intercept(invocation);
            return invocation.returnValue;
        }
        return proxy;
    };
    Proxy.prototype.defineValuePropertyProxy = function (that, interceptor, instance, propName, propValue, propDesc) {
        if (propDesc === void 0) { propDesc = { configurable: false, enumerable: true }; }
        function getProxy() {
            var method = new PropertyInfo(instance, propName);
            var invocation = new ValueGetterInvocation(method, propValue);
            interceptor.intercept(invocation);
            return invocation.returnValue;
        }
        propDesc.get = getProxy;
        function setProxy(v) {
            var method = new PropertyInfo(instance, propName);
            var invocation = new ValueSetterInvocation(method, arguments);
            interceptor.intercept(invocation);
        }
        propDesc.set = setProxy;
        this.defineProperty(that, propName, propDesc);
    };
    Proxy.prototype.defineGetSetPropertyProxy = function (that, interceptor, instance, propName, get, set, propDesc) {
        if (propDesc === void 0) { propDesc = { configurable: false, enumerable: true }; }
        function getProxy() {
            var method = new PropertyInfo(instance, propName);
            var invocation = new MethodGetterInvocation(method, get);
            interceptor.intercept(invocation);
            return invocation.returnValue;
        }
        propDesc.get = getProxy;
        function setProxy(v) {
            var method = new PropertyInfo(instance, propName);
            var invocation = new MethodSetterInvocation(method, set, arguments);
            interceptor.intercept(invocation);
        }
        propDesc.set = setProxy;
        this.defineProperty(that, propName, propDesc);
    };
    Proxy.prototype.defineProperty = function (obj, name, desc) {
        try {
            Object.defineProperty(obj, name, desc);
        }
        catch (e) {
            console.log(e.message);
        }
    };
    return Proxy;
}());

var ProxyES6 = (function () {
    function ProxyES6(interceptor) {
        this.___id = Consts.IPROXY_ID_VALUE;
        var handler = {
            apply: function (target, thisArg, argArray) {
                var funcName = Utils.functionName(target);
                var method = new MethodInfo(target, funcName);
                var invocation = new MethodInvocation(method, argArray);
                interceptor.intercept(invocation);
                return invocation.returnValue;
            },
            get: function (target, p, receiver) {
                var propValue = target[p];
                var method = new PropertyInfo(target, p, { value: true });
                var valueInvocation = new ValueGetterInvocation(method, propValue);
                interceptor.intercept(valueInvocation);
                if (valueInvocation.returnValue &&
                    valueInvocation.property.desc && valueInvocation.property.desc.value)
                    return valueInvocation.returnValue;
                else
                    return function () {
                        var args = [];
                        for (var _i = 0; _i < arguments.length; _i++) {
                            args[_i - 0] = arguments[_i];
                        }
                        interceptor.removeInvocation(valueInvocation);
                        var method = new MethodInfo(target, p);
                        var methodInvocation = new MethodInvocation(method, args);
                        interceptor.intercept(methodInvocation);
                        return methodInvocation.returnValue;
                    };
            },
            set: function (target, p, value, receiver) {
                var method = new PropertyInfo(target, p);
                var invocation = new ValueSetterInvocation(method, [value]);
                interceptor.intercept(invocation);
                return true;
            }
        };
        var p = new Proxy({}, handler);
        return p;
    }
    ProxyES6.of = function (interceptor) {
        ProxyES6.check();
        var result = new ProxyES6(interceptor);
        return result;
    };
    ProxyES6.check = function () {
        if (typeof Proxy === "undefined")
            throw new MockException(MockExceptionReason.InvalidDynamicProxyRuntime, null, "ES6 Proxy object not detected; the dynamic mocking feature requires ES6 Proxy object support");
    };
    return ProxyES6;
}());

var ProxyFactory = (function () {
    function ProxyFactory() {
    }
    ProxyFactory.createProxy = function (interceptor, instance) {
        var proxy = Proxy$1.of(instance, interceptor);
        return proxy;
    };
    ProxyFactory.createProxyES6 = function (interceptor) {
        var proxy = ProxyES6.of(interceptor);
        return proxy;
    };
    return ProxyFactory;
}());

var MockBase = (function () {
    function MockBase(targetInstance, canOverrideTarget, behavior) {
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        this.targetInstance = targetInstance;
        this.canOverrideTarget = canOverrideTarget;
        this.behavior = behavior;
        this._id = this.generateId();
        this._name = this.getNameOf(this.targetInstance);
    }
    Object.defineProperty(MockBase.prototype, "object", {
        get: function () { return this._proxy; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockBase.prototype, "name", {
        get: function () { return this._name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MockBase.prototype, "callBase", {
        get: function () { return this._callBase; },
        set: function (value) { this._callBase = value; },
        enumerable: true,
        configurable: true
    });
    MockBase.prototype.generateId = function () {
        return "Mock<" + _.uniqueId() + ">";
    };
    MockBase.prototype.getNameOf = function (instance) {
        var result;
        if (_.isFunction(instance)) {
            result = Utils.functionName(instance);
        }
        else if (_.isObject(instance)) {
            var ctor = instance.constructor;
            result = Utils.functionName(ctor);
        }
        if (result)
            result = result.trim();
        return result;
    };
    MockBase.prototype.verifyAll = function () {
        try {
            this._interceptor.verify();
        }
        catch (e) {
            throw e;
        }
    };
    MockBase.prototype.reset = function () {
        this._interceptor.reset();
    };
    return MockBase;
}());

var InterceptionAction;
(function (InterceptionAction) {
    InterceptionAction[InterceptionAction["Continue"] = 0] = "Continue";
    InterceptionAction[InterceptionAction["Stop"] = 1] = "Stop";
})(InterceptionAction || (InterceptionAction = {}));
var InterceptorContext = (function () {
    function InterceptorContext(mock) {
        this.mock = mock;
        this._actualInvocations = [];
        this._expectedCalls = [];
    }
    Object.defineProperty(InterceptorContext.prototype, "behavior", {
        get: function () { return this.mock.behavior; },
        enumerable: true,
        configurable: true
    });
    InterceptorContext.prototype.addInvocation = function (invocation) { this._actualInvocations.push(invocation); };
    InterceptorContext.prototype.removeInvocation = function (invocation) {
        _.remove(this._actualInvocations, function (x) {
            return x === invocation;
        });
    };
    InterceptorContext.prototype.actualInvocations = function () { return this._actualInvocations; };
    InterceptorContext.prototype.clearInvocations = function () { this._actualInvocations.splice(0, this._actualInvocations.length); };
    InterceptorContext.prototype.addExpectedCall = function (call) { this._expectedCalls.push(call); };
    InterceptorContext.prototype.removeExpectedCall = function (call) {
        _.filter(this._expectedCalls, function (x) {
            return x.id !== call.id;
        });
    };
    InterceptorContext.prototype.expectedCalls = function () { return this._expectedCalls; };
    InterceptorContext.prototype.clearExpectedCalls = function () { this._expectedCalls.splice(0, this._expectedCalls.length); };
    InterceptorContext.prototype.reset = function () {
        this.clearInvocations();
        this.clearExpectedCalls();
    };
    return InterceptorContext;
}());

var CurrentInterceptContext = (function () {
    function CurrentInterceptContext() {
    }
    return CurrentInterceptContext;
}());

var AddActualInvocation = (function () {
    function AddActualInvocation() {
    }
    AddActualInvocation.prototype.handleIntercept = function (invocation, ctx, localCtx) {
        ctx.addInvocation(invocation);
        return InterceptionAction.Continue;
    };
    return AddActualInvocation;
}());
var ExtractProxyCall = (function () {
    function ExtractProxyCall() {
    }
    ExtractProxyCall.prototype.handleIntercept = function (invocation, ctx, localCtx) {
        var expectedCalls = ctx.expectedCalls().slice();
        var findCallPred = function (c) { return c.matches(invocation); };
        var matchingCalls = _.filter(expectedCalls, function (c) {
            return findCallPred(c);
        });
        if (matchingCalls.length > 1)
            findCallPred = function (c) { return !c.isInvoked &&
                c.matches(invocation); };
        localCtx.call = _.find(expectedCalls, function (c) {
            return findCallPred(c);
        });
        if (localCtx.call != null) {
            localCtx.call.evaluatedSuccessfully();
        }
        else if (ctx.behavior == exports.MockBehavior.Strict) {
            throw new MockException(MockExceptionReason.NoSetup, invocation, "'" + invocation + "'");
        }
        return InterceptionAction.Continue;
    };
    return ExtractProxyCall;
}());
var ExecuteCall = (function () {
    function ExecuteCall() {
    }
    ExecuteCall.prototype.handleIntercept = function (invocation, ctx, localCtx) {
        this._ctx = ctx;
        var currentCall = localCtx.call;
        if (currentCall != null) {
            currentCall.execute(invocation);
            return InterceptionAction.Stop;
        }
        return InterceptionAction.Continue;
    };
    return ExecuteCall;
}());
var InvokeBase = (function () {
    function InvokeBase() {
    }
    InvokeBase.prototype.handleIntercept = function (invocation, ctx, localCtx) {
        if (ctx.mock.callBase) {
            invocation.invokeBase();
            return InterceptionAction.Stop;
        }
        return InterceptionAction.Continue;
    };
    return InvokeBase;
}());
var HandleMockRecursion = (function () {
    function HandleMockRecursion() {
    }
    HandleMockRecursion.prototype.handleIntercept = function (invocation, ctx, localCtx) {
        return InterceptionAction.Continue;
    };
    return HandleMockRecursion;
}());

var InterceptorExecute = (function () {
    function InterceptorExecute(mock) {
        this._interceptorContext = new InterceptorContext(mock);
    }
    Object.defineProperty(InterceptorExecute.prototype, "interceptorContext", {
        get: function () { return this._interceptorContext; },
        enumerable: true,
        configurable: true
    });
    InterceptorExecute.prototype.intercept = function (invocation) {
        var _this = this;
        var localCtx = new CurrentInterceptContext();
        _.some(this.interceptionStrategies(), function (strategy) {
            if (InterceptionAction.Stop === strategy.handleIntercept(invocation, _this.interceptorContext, localCtx)) {
                return true;
            }
        });
    };
    InterceptorExecute.prototype.removeInvocation = function (invocation) {
        this._interceptorContext.removeInvocation(invocation);
    };
    InterceptorExecute.prototype.addExpectedCall = function (call) {
        this._interceptorContext.addExpectedCall(call);
    };
    InterceptorExecute.prototype.verify = function () {
        var expectedCalls = this._interceptorContext.expectedCalls();
        var verifiableCalls = _.filter(expectedCalls, function (c) { return c.isVerifiable; });
        for (var _i = 0, verifiableCalls_1 = verifiableCalls; _i < verifiableCalls_1.length; _i++) {
            var v = verifiableCalls_1[_i];
            this.verifyCallCount(v, v.expectedCallCount);
        }
        var orderedCalls = _.filter(expectedCalls, function (c) { return c.isInSequence; });
        this.verifyCallsOrder(orderedCalls);
    };
    InterceptorExecute.prototype.verifyCallCount = function (call, times) {
        var actualCalls = this._interceptorContext.actualInvocations();
        var callCount = _.filter(actualCalls, function (c) { return call.matches(c); }).length;
        if (!times.verify(callCount))
            this.throwVerifyCallCountException(call.setupCall, times);
    };
    InterceptorExecute.prototype.throwVerifyCallCountException = function (call, times) {
        var e = new MockException(MockExceptionReason.CallCountVerificationFailed, call, times.failMessage(call));
        throw e;
    };
    InterceptorExecute.prototype.verifyCallsOrder = function (expectedCalls) {
        var actualCalls = this._interceptorContext.actualInvocations();
        this.checkCallOrderExpectations(expectedCalls, actualCalls);
    };
    InterceptorExecute.prototype.checkCallOrderExpectations = function (expectedCalls, actualCalls) {
        var checkOrder = function (expectedCallCountList) {
            var expectedCallCount = _.sum(expectedCallCountList);
            var aci = 0;
            for (var eci = 0; eci < expectedCallCountList.length; eci++) {
                var expectedCall = expectedCalls[eci];
                var expectedCallCount_1 = expectedCallCountList[eci];
                for (var count = 1; count <= expectedCallCount_1; count++) {
                    var actualCall = actualCalls[aci++];
                    if (!expectedCall.matches(actualCall))
                        return false;
                }
            }
            return aci === expectedCallCount;
        };
        var eureka = false;
        var execute = function (acc, i) {
            if (!eureka) {
                if (i === expectedCalls.length)
                    eureka = checkOrder(acc);
                else
                    for (var j = expectedCalls[i].expectedCallCount.min; j <= expectedCalls[i].expectedCallCount.max; j++) {
                        acc[i] = j;
                        execute(acc, i + 1);
                    }
            }
        };
        execute([], 0);
        if (!eureka)
            this.throwVerifyCallOrderException();
    };
    InterceptorExecute.prototype.throwVerifyCallOrderException = function () {
        var e = new MockException(MockExceptionReason.CallOrderVerificationFailed, null);
        throw e;
    };
    InterceptorExecute.prototype.reset = function () {
        this._interceptorContext.reset();
    };
    InterceptorExecute.prototype.interceptionStrategies = function () {
        var strategies = [
            new AddActualInvocation(),
            new ExtractProxyCall(),
            new ExecuteCall(),
            new InvokeBase(),
            new HandleMockRecursion()
        ];
        return strategies;
    };
    return InterceptorExecute;
}());

var InterceptorSetup = (function () {
    function InterceptorSetup() {
    }
    Object.defineProperty(InterceptorSetup.prototype, "interceptedCall", {
        get: function () { return this._interceptedCall; },
        enumerable: true,
        configurable: true
    });
    InterceptorSetup.prototype.intercept = function (invocation) {
        if (this._interceptedCall) {
            throw new MockException(MockExceptionReason.MoreThanOneSetup, invocation, "'" + invocation + "'; setup should contain only one expression");
        }
        this._interceptedCall = invocation;
    };
    InterceptorSetup.prototype.removeInvocation = function (invocation) {
        if (this._interceptedCall &&
            this._interceptedCall === invocation)
            this._interceptedCall = undefined;
    };
    return InterceptorSetup;
}());

var MethodCall = (function () {
    function MethodCall(mock, _setupExpression, interceptor, proxy) {
        this.mock = mock;
        this._setupExpression = _setupExpression;
        this._callCount = 0;
        this._id = this.generateId();
        _setupExpression(proxy);
        if (interceptor.interceptedCall) {
            var ic = interceptor.interceptedCall;
            var newArgs = this.transformToMatchers(ic.args);
            Object.defineProperty(newArgs, "callee", { configurable: false, enumerable: true, writable: false, value: ic.args.callee });
            ic.args = newArgs;
            this._setupCall = ic;
        }
        else {
            throw new MockException(MockExceptionReason.InvalidSetup, this._setupExpression, "'" + this._setupExpression + "'");
        }
    }
    MethodCall.ofStaticMock = function (mock, setupExpression) {
        var interceptor = new InterceptorSetup();
        var proxy = ProxyFactory.createProxy(interceptor, mock.targetInstance);
        var result = new MethodCall(mock, setupExpression, interceptor, proxy);
        return result;
    };
    MethodCall.ofDynamicMock = function (mock, setupExpression) {
        var interceptor = new InterceptorSetup();
        var proxy = ProxyFactory.createProxyES6(interceptor);
        var result = new MethodCall(mock, setupExpression, interceptor, proxy);
        return result;
    };
    MethodCall.prototype.generateId = function () {
        return "MethodCall<" + _.uniqueId() + ">";
    };
    MethodCall.prototype.transformToMatchers = function (args) {
        var newArgs = [];
        _.each(args, function (a) {
            if (!_.isObject(a)) {
                var newArg = new MatchValue(a);
                newArgs.push(newArg);
            }
            else {
                if (!_.isUndefined(a[Consts.IMATCH_MATCHES_NAME]) &&
                    !_.isUndefined(a[Consts.IMATCH_ID_NAME]) && a[Consts.IMATCH_ID_NAME] === Consts.IMATCH_ID_VALUE) {
                    newArgs.push(a);
                }
                else {
                    var newArg = new MatchPred(function (x) { return x === a; });
                    newArgs.push(newArg);
                }
            }
        });
        return newArgs;
    };
    Object.defineProperty(MethodCall.prototype, "id", {
        get: function () { return this._id; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "setupExpression", {
        get: function () { return this._setupExpression; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "setupCall", {
        get: function () { return this._setupCall; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "isVerifiable", {
        get: function () { return this._isVerifiable; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "isInSequence", {
        get: function () { return this._expectedCallType === exports.ExpectedCallType.InSequence; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "expectedCallCount", {
        get: function () { return this._expectedCallCount; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "isInvoked", {
        get: function () { return this._isInvoked; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MethodCall.prototype, "callCount", {
        get: function () { return this._callCount; },
        enumerable: true,
        configurable: true
    });
    MethodCall.prototype.evaluatedSuccessfully = function () {
        this._evaluatedSuccessfully = true;
    };
    MethodCall.prototype.matches = function (call) {
        var match = false;
        if (this._setupCall.property && call && call.property &&
            this._setupCall.property.name === call.property.name) {
            if (this._setupCall.args.length === call.args.length) {
                match = true;
                _.each(this.setupCall.args, function (x, index) {
                    var setupArg = x;
                    var callArg = call.args[index];
                    if (match && !setupArg.___matches(callArg))
                        match = false;
                });
            }
        }
        return match;
    };
    MethodCall.prototype.execute = function (call) {
        this._isInvoked = true;
        if (this._setupCallback != null) {
            this._setupCallback.apply(this, call.args);
        }
        if (this._thrownException != null) {
            throw this._thrownException;
        }
        this._callCount++;
    };
    MethodCall.prototype.verifiable = function (times, expectedCallType) {
        if (times === void 0) { times = Times.atLeastOnce(); }
        if (expectedCallType === void 0) { expectedCallType = exports.ExpectedCallType.InAnyOrder; }
        this._isVerifiable = true;
        this._expectedCallCount = times;
        this._expectedCallType = expectedCallType;
    };
    return MethodCall;
}());

var __extends$3 = (undefined && undefined.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var MethodCallReturn = (function (_super) {
    __extends$3(MethodCallReturn, _super);
    function MethodCallReturn(mock, setupExpression, interceptor, proxy) {
        var _this = _super.call(this, mock, setupExpression, interceptor, proxy) || this;
        _this._overrideTarget = mock.canOverrideTarget;
        return _this;
    }
    MethodCallReturn.ofStaticMock = function (mock, setupExpression) {
        var interceptor = new InterceptorSetup();
        var proxy = ProxyFactory.createProxy(interceptor, mock.targetInstance);
        var result = new MethodCallReturn(mock, setupExpression, interceptor, proxy);
        return result;
    };
    MethodCallReturn.ofDynamicMock = function (mock, setupExpression) {
        var interceptor = new InterceptorSetup();
        var proxy = ProxyFactory.createProxyES6(interceptor);
        var result = new MethodCallReturn(mock, setupExpression, interceptor, proxy);
        return result;
    };
    MethodCallReturn.prototype.execute = function (call) {
        _super.prototype.execute.call(this, call);
        if (this._callBase)
            call.invokeBase();
        else if (this.hasReturnValue) {
            call.returnValue = this._returnValueFunc.apply(this, call.args);
            call.property.desc = { value: this.setupCall.property.desc && this.setupCall.property.desc.value };
        }
    };
    MethodCallReturn.prototype.callback = function (action) {
        this._setupCallback = action;
        return this;
    };
    MethodCallReturn.prototype.throws = function (exception) {
        this._thrownException = exception;
        return this;
    };
    MethodCallReturn.prototype.returns = function (valueFunc) {
        this._returnValueFunc = valueFunc;
        this.hasReturnValue = true;
        if (this._overrideTarget) {
            var obj = this.mock.targetInstance;
            var name_1 = this.setupCall.property.name;
            var desc = this.setupCall.property.desc;
            if (desc) {
                desc.value = this._returnValueFunc;
                Object.defineProperty(obj, name_1, desc);
            }
        }
        return this;
    };
    MethodCallReturn.prototype.callBase = function () {
        this._callBase = true;
        return this;
    };
    return MethodCallReturn;
}(MethodCall));

var __extends = (undefined && undefined.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var StaticMock = (function (_super) {
    __extends(StaticMock, _super);
    function StaticMock(targetInstance, canOverrideTarget, behavior) {
        var _this = _super.call(this, targetInstance, canOverrideTarget, behavior) || this;
        _this._interceptor = new InterceptorExecute(_this);
        _this._proxy = ProxyFactory.createProxy(_this._interceptor, targetInstance);
        return _this;
    }
    StaticMock.ofInstance = function (targetInstance, behavior) {
        targetInstance = StaticMock.cloneDeep(targetInstance);
        var mock = new StaticMock(targetInstance, true, behavior);
        return mock;
    };
    StaticMock.ofGlobalInstance = function (targetInstance, behavior) {
        var mock = new StaticMock(targetInstance, false, behavior);
        return mock;
    };
    StaticMock.ofType = function (targetConstructor, behavior, targetConstructorArgs) {
        var targetInstance = Utils.conthunktor(targetConstructor, targetConstructorArgs);
        var mock = new StaticMock(targetInstance, true, behavior);
        return mock;
    };
    StaticMock.cloneDeep = function (target) {
        var copy = target;
        if (!_.isFunction(target)) {
            var func = function (x) {
                var value;
                if (Proxy$1.isProxy(x))
                    return x;
            };
            copy = _.cloneDeepWith(target, func);
        }
        return copy;
    };
    StaticMock.prototype.setup = function (expression) {
        var call = MethodCallReturn.ofStaticMock(this, expression);
        this._interceptor.addExpectedCall(call);
        return call;
    };
    StaticMock.prototype.verify = function (expression, times) {
        var call = MethodCall.ofStaticMock(this, expression);
        this._interceptor.addExpectedCall(call);
        try {
            this._interceptor.verifyCallCount(call, times);
        }
        catch (e) {
            throw e;
        }
    };
    return StaticMock;
}(MockBase));

var __extends$4 = (undefined && undefined.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var DynamicMock = (function (_super) {
    __extends$4(DynamicMock, _super);
    function DynamicMock(name, canOverrideTarget, behavior) {
        var _this = _super.call(this, {}, canOverrideTarget, behavior) || this;
        if (name)
            _this._name = name;
        _this._interceptor = new InterceptorExecute(_this);
        _this._proxy = ProxyFactory.createProxyES6(_this._interceptor);
        return _this;
    }
    DynamicMock.ofType = function (name, behavior) {
        var mock = new DynamicMock(name, true, behavior);
        return mock;
    };
    DynamicMock.prototype.setup = function (expression) {
        var call = MethodCallReturn.ofDynamicMock(this, expression);
        this._interceptor.addExpectedCall(call);
        return call;
    };
    DynamicMock.prototype.verify = function (expression, times) {
        var call = MethodCall.ofDynamicMock(this, expression);
        this._interceptor.addExpectedCall(call);
        try {
            this._interceptor.verifyCallCount(call, times);
        }
        catch (e) {
            throw e;
        }
    };
    return DynamicMock;
}(MockBase));

var MockApi = (function () {
    function MockApi() {
    }
    MockApi.ofInstance = function (targetInstance, behavior) {
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var mock = StaticMock.ofInstance(targetInstance, behavior);
        return mock;
    };
    MockApi.ofType = function (targetConstructor, behavior) {
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var targetConstructorArgs = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            targetConstructorArgs[_i - 2] = arguments[_i];
        }
        var mock;
        if (targetConstructor)
            mock = MockApi.ofType2(targetConstructor, targetConstructorArgs, behavior);
        else
            mock = DynamicMock.ofType(undefined, behavior);
        return mock;
    };
    MockApi.ofType2 = function (targetConstructor, targetConstructorArgs, behavior) {
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var mock = StaticMock.ofType(targetConstructor, behavior, targetConstructorArgs);
        return mock;
    };
    return MockApi;
}());

var It = (function () {
    function It() {
    }
    It.isValue = function (x) {
        var matcher = new MatchValue(x);
        return matcher;
    };
    It.isAnyObject = function (x) {
        var matcher = new MatchAnyObject(x);
        return matcher;
    };
    It.isAny = function () {
        var matcher = new MatchAny();
        return matcher;
    };
    It.isAnyString = function () {
        var matcher = new MatchAnyString();
        return matcher;
    };
    It.isAnyNumber = function () {
        var matcher = new MatchAnyNumber();
        return matcher;
    };
    It.is = function (predicate) {
        var matcher = new MatchPred(predicate);
        return matcher;
    };
    return It;
}());

var GlobalType;
(function (GlobalType) {
    GlobalType[GlobalType["Class"] = 0] = "Class";
    GlobalType[GlobalType["Function"] = 1] = "Function";
    GlobalType[GlobalType["Value"] = 2] = "Value";
})(GlobalType || (GlobalType = {}));
var GlobalMock = (function () {
    function GlobalMock(mock, _name, _type, container) {
        this.mock = mock;
        this._name = _name;
        this._type = _type;
        this.container = container;
        if (!this._name)
            this._name = mock.name;
    }
    Object.defineProperty(GlobalMock.prototype, "object", {
        get: function () { return this.mock.object; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GlobalMock.prototype, "name", {
        get: function () { return this._name || this.mock.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GlobalMock.prototype, "behavior", {
        get: function () { return this.mock.behavior; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GlobalMock.prototype, "callBase", {
        get: function () { return this.mock.callBase; },
        set: function (value) { this.mock.callBase = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(GlobalMock.prototype, "type", {
        get: function () { return this._type; },
        enumerable: true,
        configurable: true
    });
    GlobalMock.prototype.setup = function (expression) {
        return this.mock.setup(expression);
    };
    GlobalMock.prototype.verify = function (expression, times) {
        this.mock.verify(expression, times);
    };
    GlobalMock.prototype.verifyAll = function () {
        this.mock.verifyAll();
    };
    GlobalMock.prototype.reset = function () {
        this.mock.reset();
    };
    return GlobalMock;
}());

var GlobalMockApi = (function () {
    function GlobalMockApi() {
    }
    GlobalMockApi.ofInstance = function (targetInstance, globalName, container, behavior) {
        if (container === void 0) { container = window; }
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var mock = StaticMock.ofGlobalInstance(targetInstance, behavior);
        var type = _.isFunction(targetInstance) ? GlobalType.Function : GlobalType.Value;
        return new GlobalMock(mock, globalName, type, container);
    };
    GlobalMockApi.ofType = function (targetConstructor, container, behavior) {
        if (container === void 0) { container = window; }
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var targetInstance = new targetConstructor();
        var mock = StaticMock.ofInstance(targetInstance, behavior);
        return new GlobalMock(mock, undefined, GlobalType.Class, container);
    };
    GlobalMockApi.ofType2 = function (globalName, container, behavior) {
        if (container === void 0) { container = window; }
        if (behavior === void 0) { behavior = exports.MockBehavior.Loose; }
        var mock = DynamicMock.ofType(globalName, behavior);
        return new GlobalMock(mock, undefined, GlobalType.Class, container);
    };
    return GlobalMockApi;
}());

var GlobalScope = (function () {
    function GlobalScope(_args) {
        this._args = _args;
    }
    GlobalScope.prototype.with = function (action) {
        var initial = {};
        try {
            _.each(this._args, function (a) {
                var containerProps = PropertyRetriever.getOwnAndPrototypeEnumerablesAndNonenumerables(a.container);
                var prop = _.find(containerProps, function (p) { return p.name === a.name; });
                if (prop) {
                    initial[a.name] = prop.desc;
                    var desc = {};
                    switch (a.type) {
                        case GlobalType.Class:
                            desc.value = function () { return a.mock.object; };
                            break;
                        case GlobalType.Function:
                            desc.value = a.mock.object;
                            break;
                        case GlobalType.Value:
                            desc.get = function () { return a.mock.object; };
                            break;
                        default:
                            throw new MockException(MockExceptionReason.UnknownGlobalType, a, "unknown global type: " + a.type);
                    }
                    try {
                        Object.defineProperty(a.container, a.name, desc);
                    }
                    catch (e) {
                        console.log("1: " + e);
                    }
                }
            });
            action.apply(this, this._args);
        }
        catch (e) {
            console.log("2: " + e);
        }
        finally {
            _.each(this._args, function (a) {
                var desc = initial[a.name];
                if (desc) {
                    switch (a.type) {
                        case GlobalType.Class:
                            break;
                        case GlobalType.Function:
                            break;
                        case GlobalType.Value:
                            desc.configurable = true;
                            break;
                        default:
                    }
                    try {
                        Object.defineProperty(a.container, a.name, desc);
                    }
                    catch (e) {
                        console.log("3: " + e);
                    }
                }
            });
        }
    };
    return GlobalScope;
}());

var GlobalScopeApi = (function () {
    function GlobalScopeApi() {
    }
    GlobalScopeApi.using = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i - 0] = arguments[_i];
        }
        var scope = new GlobalScope(args);
        return scope;
    };
    return GlobalScopeApi;
}());

exports.Mock = MockApi;
exports.It = It;
exports.Times = Times;
exports.GlobalMock = GlobalMockApi;
exports.GlobalScope = GlobalScopeApi;
exports.MockException = MockException;

Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=typemoq.js.map
